@@ -17,7 +17,7 @@ int s6_svc_write (char const *fifo, char const *data, size_t datalen)
     default : return -1 ;
   }
   if (ndelay_off(fd) == -1) return -1 ;
-  if (fd_write(fd, data, datalen) == -1)
+  if (datalen && fd_write(fd, data, datalen) == -1)
   {
     fd_close(fd) ;
     return -1 ;
