@@ -1,24 +1,25 @@
 /* ISC license. */
 
+#include <errno.h>
+
 #include <skalibs/allreadwrite.h>
 #include <skalibs/strerr2.h>
-#include <skalibs/env.h>
 #include <skalibs/djbunix.h>
 
 #define USAGE "s6lockd-helper lockfile"
 #define dieusage() strerr_dieusage(100, USAGE)
 
-int main (int argc, char const *const *argv, char const *const *envp)
+int main (int argc, char const *const *argv)
 {
-  int fd ;
-  char const *x = env_get2(envp, "S6LOCK_EX") ;
+  int fd, r ;
   char c ;
   PROG = "s6lockd-helper" ;
-  if (argc < 2) dieusage() ;
-  fd = open_create(argv[1]) ;
+  if (argc < 3) dieusage() ;
+  fd = open_create(argv[2]) ;
   if (fd < 0) strerr_diefu2sys(111, "open ", argv[1]) ;
-  if (((x && *x) ? lock_ex(fd) : lock_sh(fd)) < 0)
-    strerr_diefu2sys(111, "lock ", argv[1]) ;
+  r = fd_lock(fd, argv[1][0] == 'w', 0) ;
+  if (!r) errno = EBUSY ;
+  if (r < 1) strerr_diefu2sys(111, "lock ", argv[2]) ;
   if (fd_write(1, "!", 1) <= 0)
     strerr_diefu1sys(111, "write to stdout") ;
   if (fd_read(0, &c, 1) < 0)
